<?php
namespace Controllers;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Notification;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use Exception;

class Gop00240PreparacaoDoOrcamento extends \Teknisa\GER\Controllers\TeknisaCrud
{
    
    protected $dataSourceName = 'lista';
    protected $crudService;
    protected $parameterBag;
    protected $dataSourceManager;
    protected $novoCodigo;
    protected $entityManager;
    protected $connection;
    protected $util;
    
    
    public function __construct($crudService, $parameterBag, $dataSourceManager, $entityManager, $novoCodigo, $environment, $connection, $util)
    {
        $this->crudService       = $crudService;
        $this->parameterBag      = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->entityManager     = $entityManager;
        $this->novoCodigo        = $novoCodigo;
        $this->environment       = $environment;
        $this->connection        = $connection;
        $this->util              = $util;
    }
    
    public function getDados(DTO\Request\Row $request, DTO\Response $response)
    {
        $rows         = $request->getRow();
        $nrOrg        = $this->environment->getNrOrgTrab();
        $currentTime  = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        $cdOperLogado = $this->environment->getCdOperador();
        $cdFilial = [];
        
        if (!is_string($rows['CDFILIAL'])) {
            $row = $rows['CDFILIAL'];
            $cdFilial = $rows['CDFILIAL'];
        } else {
            $row = $this->selectUnidade($rows);
            $cdFilial = array_column($row, 'CDFILIAL');
        }
        
        $nrListaFilial = $this->util->insertLista($cdFilial);
        $nrListaOperador = $this->util->insertLista($rows['CDOPERADOR']);
        
        $ano             = $rows['DTANO'];
        $numPeriodo      = $rows['VALUE_PERIOD'];
        $idExclui        = $rows['IDEXCLUIR'];
        $listaOper       = 'S';
        $listaFilial     = 'S';
        $orcadeta        = 'S';
        $itemorca        = $rows['stringTpOrca'];
        
        try{
            
            $this->PREPARA_ORCAMENTO($ano, $numPeriodo, $nrListaOperador, $nrListaFilial, $idExclui, $listaOper, $listaFilial, $orcadeta, $itemorca, $cdOperLogado);
            
            $naoGerados = array();
            
            $i = sizeof($cdFilial);
            for ($x = 0; $i > $x; $i--){
                array_push($naoGerados, $this->naoGerados($rows, $cdFilial[$i-1]));
            }
            
            if($naoGerados){
                $erroList = array();
                
                foreach ($naoGerados as $values) {
                    foreach($values as $value){
                        $erroList[$value['CDFILIAL']] = $erroList[$value['CDFILIAL']] ?? ['ERROS' => []];
                        $erroList[$value['CDFILIAL']]['NMFILIAL'] = $value['NMFILIAL'];
                        $erroList[$value['CDFILIAL']]['ERROS'][] = $value['DSMOTIVO'];
                    }
                }
                unset($value);
                $response->addDataSet(new DataSet("errorList", $erroList));
            }
            $response->addDataSet(new DataSet("ROW", $row));
        }catch(\Exception $ex){
            $response->setError(new Error("Nao foi possivel processar os dados enviados --- " . $ex->getMessage(), 500));
        }
    }
    
    public function selectUnidade($rows) {
        $select = "SELECT
                        '' AS IDMARCADO,
                        FL.CDFILIAL,
                        FL.NMFILIAL,
                        FL.NRORG
                    FROM
                        FILIAL         FL,
                        FILGRUPFILI    FG,
                        FILIALOPEROP   FO
                    WHERE
                        ( FL.CDFILIAL = FG.CDFILIAL (+) )
                        AND ( FO.CDFILIAL = FL.CDFILIAL )
                        AND ( ( INSTR(:P_CDGRUPFILI, FG.CDGRUPFILI) > 0 )
                              OR ( 'T' = NVL(:P_CDGRUPFILI, 'T') ) )
                        AND ( ( INSTR(:P_CDSEGMER, FL.CDSEGMER) > 0 )
                              OR ( 'T' = NVL(:P_CDSEGMER, 'T') ) )
                        AND ( ( INSTR(:P_CDOPERADOR, FO.CDOPERADOR) > 0 )
                              OR ( 'T' = NVL(:P_CDOPERADOR, 'T') ) )
                        AND ( (FL.CDSUPERVISOR = :P_CDSUPERVISOR)
                              OR ( 'T' = NVL(:P_CDSUPERVISOR, 'T') ) )
                        AND ( (FL.CDREGIAO = :P_CDREGIAO ) 
                              OR ( 'T' = NVL(:P_CDREGIAO, 'T') ) )
                    GROUP BY
                        FL.CDFILIAL,
                        FL.NMFILIAL,
                        FL.NRORG
                    ORDER BY
                        FL.CDFILIAL";
                        
        $row = $rows == null ? false : true;                
        $params = array (
            'P_CDGRUPFILI'      => $row ? (isset($rows['P_CDGRUPFILI']) ? $rows['P_CDGRUPFILI'] : 'T') : null,
            'P_CDSEGMER'        => $row ? (isset($rows['P_CDSEGMER']) ? $rows['P_CDSEGMER'] : 'T') : null,
            'P_CDOPERADOR'      => $row ? (isset($rows['P_CDOPERADOR']) ? $rows['P_CDOPERADOR'] : 'T') : null,
            'P_CDSUPERVISOR'    => $row ? (isset($rows['P_CDSUPERVISOR']) ? $rows['P_CDSUPERVISOR'] : 'T') : null,
            'P_CDREGIAO'        => $row ? (isset($rows['P_CDREGIAO']) ? $rows['P_CDREGIAO'] : 'T') : null
        );
         
        try{
            $std = $this->connection->prepare($select);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result;
            
        }catch (Exception $e) {
            throw $e;
        }
    }
    
    public function PREPARA_ORCAMENTO($ano, $numPeriodo, $nrListaOperador, $nrListaUnidade, $idExclui, $listaOper, $listaFilial, $orcadeta, $itemorca, $cdOperLogado)
    {
        $procedure = new StoredProcedure($this->connection, 'PREPARA_ORCAMENTO');
        $procedure->addParam(new Param('P_NRANOORCAOP', Param::PARAM_INPUT, $ano, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDVERSORCAOP', Param::PARAM_INPUT, $numPeriodo, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $nrListaOperador, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $nrListaUnidade, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDEXCLUI', Param::PARAM_INPUT, $idExclui, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LISTAOPER', Param::PARAM_INPUT, $listaOper, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LISTAFIL', Param::PARAM_INPUT, $listaFilial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ORCADETA', Param::PARAM_INPUT, $orcadeta, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ITEMORCA', Param::PARAM_INPUT, $itemorca, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERLOGADO', Param::PARAM_INPUT, $cdOperLogado, Param::PARAM_TYPE_STR));
    
        $procedure->execute();
     
    }
    
    public function naoGerados($row, $cdfilial){
        $parametros =  array(
                            'P_CDFILIAL' =>  $cdfilial,
                            'P_NRANOORCAGOP' => $row['DTANO'],
                            'P_IDPERIODOORCA'=> $row['IDPERIODOORCA'],
                            'P_NRPERIODOORCA'=> $row['VALUE_PERIOD']
                        );
        $stmt = $this->connection->prepare("SELECT L.*, F.NMFILIAL FROM LOGPREPARAORCAGOP L, FILIAL F
             WHERE
               L.CDFILIAL = F.CDFILIAL AND
               L.NRANOORCAGOP = :P_NRANOORCAGOP AND
               L.IDPERIODOORCA = :P_IDPERIODOORCA AND
               L.NRPERIODOORCA = :P_NRPERIODOORCA AND
               L.CDFILIAL   = :P_CDFILIAL"
               ); 
        $stmt->execute($parametros);
        
        $result = $stmt->fetchAll();
        
		return  $result;
    }
    
    public function searchItens(DTO\Request\Filter $request, DTO\Response $response){
        
         $query = "SELECT IDTIPOORCAOP, NRITEMOPERACIONAL,
            CASE NRITEMOPERACIONAL
            WHEN 0 THEN 'Faturamento'
            WHEN 1 THEN 'Imposto'
            WHEN 2 THEN 'Consumo'
            WHEN 3 THEN 'Mão de Obra'
            WHEN 4 THEN 'Taxa Administrativa'
            WHEN 5 THEN 'Desp. Matriz'
            WHEN 6 THEN 'Desp. Diversas'
            WHEN 7 THEN 'Desp. Lanc. Entrada' 
            WHEN 8 THEN 'Imposto a Recuperar'
            WHEN 9 THEN 'Imposto Sobre o Lucro'
            WHEN 10 THEN 'Investimento'
            WHEN 11 THEN 'Depreciações'
            ELSE 'Invalido'
            END
            NMORCAMENTO
            FROM ITEMOPERACIONAL WHERE IDCHECK= 'S' ORDER BY NRITEMOPERACIONAL";
        
        try{
            $stmt = $this -> connection -> prepare($query);
            
            $stmt -> execute();
            
            $result =  $stmt -> fetchAll();
            
            $response ->addDataSet(new DataSet("DATA", $result));
            
        }catch(Exception $e){
            throw $e;
        }
        
    }

}