/* Sistemas Integrados TECNISA

Planejamento de Alimentao Industrial
   
Nome: 2500_PLATRGS_NUTRIENTES_CALORIAS

Descrio:
A trigger ir atualizar os campos 'Compe Clculo de Caloria', 'Caloria' e 'Fator de Cculo de Caloria'
(Fator Nutricional) da tabela de nutrientes sempre que forem informados novos nutrietes na tela
Parmetros de Planejamento, aba Geral, aba Cardpio.

Criao:
09/2014 - Rodrigo Oliveira
*/

IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
           WHERE O.NAME = 'NUTRIENTES_CALORIAS' AND
                 TYPE   = 'TR' 		        AND
	              U.UID  = O.UID             AND
	              U.NAME = USER)
DROP TRIGGER NUTRIENTES_CALORIAS
GO

CREATE TRIGGER NUTRIENTES_CALORIAS
ON PARAMGERAL
FOR UPDATE, INSERT 
AS
DECLARE
   @V_INS_CDCALORIAS VARCHAR(3),
   @V_INS_CDCARBOHIDRA VARCHAR(3),
   @V_INS_CDPROTEINA VARCHAR(3),
   @V_INS_CDLIPIDIOS  VARCHAR(3),
   @V_INS_CDALCOOL VARCHAR(3),
   @V_DEL_CDCALORIAS VARCHAR(3),
   @V_DEL_CDCARBOHIDRA VARCHAR(3),
   @V_DEL_CDPROTEINA VARCHAR(3),
   @V_DEL_CDLIPIDIOS  VARCHAR(3),
   @V_DEL_CDALCOOL VARCHAR(3);
BEGIN
   /*Busca os novos cdigos de nutrientes da paramgeral*/	
   SELECT
      @V_INS_CDCALORIAS = CDCALORIAS,
      @V_INS_CDCARBOHIDRA = CDCARBOHIDRA,
      @V_INS_CDPROTEINA = CDPROTEINA,
      @V_INS_CDLIPIDIOS = CDLIPIDIOS,
      @V_INS_CDALCOOL = CDALCOOL
   FROM
      INSERTED;
   
   /*Busca os antigos cdigos de nutrientes da paramgeral*/   
   SELECT
      @V_DEL_CDCALORIAS = CDCALORIAS,
      @V_DEL_CDCARBOHIDRA = CDCARBOHIDRA,
      @V_DEL_CDPROTEINA = CDPROTEINA,
      @V_DEL_CDLIPIDIOS = CDLIPIDIOS,
      @V_DEL_CDALCOOL = CDALCOOL
   FROM
      DELETED;
      
   /*Verifica se algum dos nutrientes da paramgeral foi modificado*/
   IF (ISNULL(@V_INS_CDCALORIAS, '0000') <> ISNULL(@V_DEL_CDCALORIAS, '0000'))     OR
      (ISNULL(@V_INS_CDCARBOHIDRA, '0000') <> ISNULL(@V_DEL_CDCARBOHIDRA, '0000')) OR
      (ISNULL(@V_INS_CDPROTEINA, '0000') <> ISNULL(@V_DEL_CDPROTEINA, '0000'))     OR
      (ISNULL(@V_INS_CDLIPIDIOS, '0000') <> ISNULL(@V_DEL_CDLIPIDIOS, '0000'))     OR
      (ISNULL(@V_INS_CDALCOOL, '0000') <> ISNULL(@V_DEL_CDALCOOL, '0000'))         BEGIN
      
      /*Limpa os dados do nutriente caloria, dos nutrientes que compem o clculo da caloria e do fator nutricional na tabela de nutrientes*/
      UPDATE NUTRIENTE
      SET IDCALORIAS = 'N',
          IDKCAL = 'N',
          VRFATORNUTR = NULL;
      
      /*Atualiza os nutrientes carboidrado e proteina para os mesmos comporem o clculo de caloria e terem o fator nutricional = 4*/    
      UPDATE NUTRIENTE
      SET IDCALORIAS = 'S',
          VRFATORNUTR = 4
      WHERE (CDNUTRIENTE = @V_INS_CDCARBOHIDRA) OR
            (CDNUTRIENTE = @V_INS_CDPROTEINA);
      
      /*Atualiza o nutriente lipidio para o mesmo compor o clculo de caloria e ter o fator nutricional = 9*/
      UPDATE NUTRIENTE
      SET IDCALORIAS = 'S',
          VRFATORNUTR = 9
      WHERE CDNUTRIENTE = @V_INS_CDLIPIDIOS;
      
      /*Atualiza o nutriente lcool para o mesmo compor o clculo de caloria e ter o fator nutricional = 7*/
      UPDATE NUTRIENTE
      SET IDCALORIAS = 'S',
          VRFATORNUTR = 7
      WHERE CDNUTRIENTE = @V_INS_CDALCOOL;
      
      /*Atualiza o nutriente caloria para o mesmo ser marcado como caloria na tabela nutriente*/
      UPDATE NUTRIENTE
      SET IDKCAL = 'S'
      WHERE CDNUTRIENTE = @V_INS_CDCALORIAS;

   END;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'NUTRIENTES_CALORIAS','5.80.xxx';
GO